@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.29 2003/05/11 15:57:37 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.36 2003/05/27 20:01:49 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -33,6 +33,7 @@
 /* gtk+/gnome */
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <gconf/gconf-client.h>
 /* xine */
 #include <xine.h>
 
@@ -50,8 +51,8 @@
 #   define N_(String) (String)
 #endif
 
-#define DEFAULT_HEIGHT 420
-#define DEFAULT_WIDTH 315
+#define DEFAULT_HEIGHT 315
+#define DEFAULT_WIDTH 420
 #define CONFIG_FILE ".gnome2"G_DIR_SEPARATOR_S"totem_config"
 #define DEFAULT_TITLE _("Totem Video Window")
 
@@ -86,7 +87,6 @@ typedef struct {
 enum {
 	PROP_0,
 	PROP_LOGO_MODE,
-	PROP_FULLSCREEN,
 	PROP_SPEED,
 	PROP_POSITION,
 	PROP_CURRENT_TIME,
@@ -141,14 +141,11 @@ struct BaconVideoWidgetPrivate {
 
 	GAsyncQueue *queue;
 	int video_width, video_height;
+	int init_width, init_height;
 
 	/* fullscreen stuff */
 	gboolean fullscreen_mode;
-	GdkWindow *fullscreen_window;
-	GdkRectangle fullscreen_rect;
 	gboolean cursor_shown;
-	gboolean pml;
-	ScreenSaver *scr;
 };
 
 static const char *mms_bandwidth_strs[]={"14.4 Kbps (Modem)",
@@ -182,6 +179,8 @@ static gboolean bacon_video_widget_button_press (GtkWidget *widget,
 static gboolean bacon_video_widget_key_press (GtkWidget *widget,
 		GdkEventKey *event);
 
+static void bacon_video_widget_size_request (GtkWidget *widget,
+		GtkRequisition *requisition);
 static void bacon_video_widget_size_allocate (GtkWidget *widget,
 		GtkAllocation *allocation);
 static xine_vo_driver_t * load_video_out_driver (BaconVideoWidget *bvw,
@@ -194,7 +193,6 @@ static GtkWidgetClass *parent_class = NULL;
 
 static void xine_event (void *user_data, const xine_event_t *event);
 static gboolean bacon_video_widget_idle_signal (BaconVideoWidget *bvw);
-static void size_changed_cb (GdkScreen *screen, BaconVideoWidget *bvw);
 
 static int bvw_table_signals[LAST_SIGNAL] = { 0 };
 
@@ -239,6 +237,7 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 	/* GtkWidget */
 	widget_class->realize = bacon_video_widget_realize;
 	widget_class->unrealize = bacon_video_widget_unrealize;
+	widget_class->size_request = bacon_video_widget_size_request;
 	widget_class->size_allocate = bacon_video_widget_size_allocate;
 	widget_class->expose_event = bacon_video_widget_expose;
 	widget_class->motion_notify_event = bacon_video_widget_motion_notify;
@@ -254,9 +253,6 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 	g_object_class_install_property (object_class, PROP_LOGO_MODE,
 			g_param_spec_boolean ("logo_mode", NULL, NULL,
 				FALSE, G_PARAM_READWRITE));
-	g_object_class_install_property (object_class, PROP_FULLSCREEN,
-			g_param_spec_boolean ("fullscreen", NULL, NULL,
-				FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (object_class, PROP_SPEED,
 			g_param_spec_int ("speed", NULL, NULL,
 				SPEED_PAUSE, SPEED_NORMAL,
@@ -353,6 +349,7 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	GtkWidget *widget = (GtkWidget *) bvw;
 	const char *const *autoplug_list;
 	int i = 0;
+	GConfClient *gc;
 
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
@@ -360,15 +357,14 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (bvw), GTK_CAN_FOCUS);
 	GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (bvw), GTK_DOUBLE_BUFFERED);
 
-	/* Set the default size to be a 4:3 ratio */
-	widget->requisition.width = DEFAULT_HEIGHT;
-	widget->requisition.height = DEFAULT_WIDTH;
-
 	bvw->priv = g_new0 (BaconVideoWidgetPrivate, 1);
 	bvw->priv->xine = xine_new ();
 	bvw->priv->cursor_shown = TRUE;
 	bvw->priv->vis_name = g_strdup ("goom");
 
+	bvw->priv->init_width = 0;
+	bvw->priv->init_height = 0;
+
 	bvw->priv->queue = g_async_queue_new ();
 
 	/* init configuration  */
@@ -376,15 +372,18 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 
 	xine_init (bvw->priv->xine);
 
-#ifndef DEBUG
-	xine_engine_set_param (bvw->priv->xine,
-			XINE_ENGINE_PARAM_VERBOSITY,
-			XINE_VERBOSITY_NONE);
-#else
-	xine_engine_set_param (bvw->priv->xine,
-			XINE_ENGINE_PARAM_VERBOSITY,
-			XINE_VERBOSITY_DEBUG);
-#endif
+	/* Debug configuration */
+	gc = gconf_client_get_default ();
+	if (gconf_client_get_bool (gc, GCONF_PREFIX"/debug", NULL) == FALSE)
+	{
+		xine_engine_set_param (bvw->priv->xine,
+				XINE_ENGINE_PARAM_VERBOSITY,
+				XINE_VERBOSITY_NONE);
+	} else {
+		xine_engine_set_param (bvw->priv->xine,
+				XINE_ENGINE_PARAM_VERBOSITY,
+				XINE_VERBOSITY_DEBUG);
+	}
 
 	/* Can we play DVDs and VCDs ? */
 	autoplug_list = xine_get_autoplay_input_plugin_ids (bvw->priv->xine);
@@ -442,15 +441,8 @@ dest_size_cb (void *bvw_gen,
 		video_height = video_height * bvw->priv->display_ratio
 			/ video_pixel_aspect + .5;
 
-	if (bvw->priv->fullscreen_mode)
-	{
-		*dest_width = bvw->priv->fullscreen_rect.width;
-		*dest_height = bvw->priv->fullscreen_rect.height;
-	} else {
-		*dest_width = GTK_WIDGET(bvw)->allocation.width;
-		*dest_height = GTK_WIDGET(bvw)->allocation.height;
-	}
-
+	*dest_width = GTK_WIDGET(bvw)->allocation.width;
+	*dest_height = GTK_WIDGET(bvw)->allocation.height;
 	*dest_pixel_aspect = bvw->priv->display_ratio;
 }
 
@@ -481,33 +473,26 @@ frame_output_cb (void *bvw_gen,
 	*win_x = bvw->priv->xpos;
 	*win_y = bvw->priv->ypos;
 
-	if (bvw->priv->fullscreen_mode)
+	*dest_width = GTK_WIDGET(bvw)->allocation.width;
+	*dest_height = GTK_WIDGET(bvw)->allocation.height;
+
+	/* Size changed */
+	if (bvw->priv->video_width != video_width
+			|| bvw->priv->video_height != video_height)
 	{
-		*dest_width = bvw->priv->fullscreen_rect.width;
-		*dest_height = bvw->priv->fullscreen_rect.height;
-	} else {
-		*dest_width = GTK_WIDGET(bvw)->allocation.width;
-		*dest_height = GTK_WIDGET(bvw)->allocation.height;
+		bvw->priv->video_width = video_width;
+		bvw->priv->video_height = video_height;
 
-		/* Size changed */
-		if (bvw->priv->video_width != video_width
-				|| bvw->priv->video_height != video_height)
+		if (bvw->priv->auto_resize == TRUE
+				&& bvw->priv->logo_mode == FALSE)
 		{
-			bvw->priv->video_width = video_width;
-			bvw->priv->video_height = video_height;
+			signal_data *data;
 
-			if (bvw->priv->auto_resize == TRUE
-					&& bvw->priv->logo_mode == FALSE)
-			{
-				signal_data *data;
-
-				data = g_new0 (signal_data, 1);
-				data->signal = RATIO;
-				g_async_queue_push (bvw->priv->queue, data);
-				g_idle_add ((GSourceFunc)
-						bacon_video_widget_idle_signal,
-						bvw);
-			}
+			data = g_new0 (signal_data, 1);
+			data->signal = RATIO;
+			g_async_queue_push (bvw->priv->queue, data);
+			g_idle_add ((GSourceFunc)
+					bacon_video_widget_idle_signal, bvw);
 		}
 	}
 
@@ -626,31 +611,10 @@ load_audio_out_driver (BaconVideoWidget *bvw, GError **err)
 	return ao_driver;
 }
 
-static void
-update_fullscreen_size (BaconVideoWidget *bvw)
-{
-	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
-			gdk_screen_get_monitor_at_window
-			(gdk_screen_get_default (),
-			 bvw->priv->video_window),
-			&bvw->priv->fullscreen_rect);
-}
-
-static void
-size_changed_cb (GdkScreen *screen, BaconVideoWidget *bvw)
-{
-	update_fullscreen_size (bvw);
-	if (bvw->priv->fullscreen_mode)
-	{
-		gdk_window_resize (bvw->priv->fullscreen_window,
-				bvw->priv->fullscreen_rect.width,
-				bvw->priv->fullscreen_rect.height);
-	}
-}
-
 static void
 setup_config (BaconVideoWidget *bvw)
 {
+	GConfClient *gc;
 	char *path;
 	xine_cfg_entry_t entry;
 	const char *demux_strategies[] = {"default", "reverse", "content",
@@ -680,6 +644,81 @@ setup_config (BaconVideoWidget *bvw)
 			50, 0, 100, "amp volume level",
 			NULL, 10, NULL, NULL);
 	bvw->priv->volume = -1;
+
+	/* Proxy configuration */
+	gc = gconf_client_get_default ();
+	if (gc == NULL)
+		return;
+
+	memset (&entry, 0, sizeof (entry));
+
+	if (gconf_client_get_bool (gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
+	{
+		if (xine_config_lookup_entry (bvw->priv->xine,
+				"input.http_proxy_host", &entry))
+		{
+			entry.str_value = "";
+			xine_config_update_entry (bvw->priv->xine, &entry);
+		}
+
+		return;
+	}
+
+	memset (&entry, 0, sizeof (entry));
+	if (xine_config_lookup_entry (bvw->priv->xine,
+			"input.http_proxy_host", &entry))
+	{
+		entry.str_value = gconf_client_get_string (gc,
+				"/system/http_proxy/host", NULL);
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
+
+	memset (&entry, 0, sizeof (entry));
+	if (xine_config_lookup_entry (bvw->priv->xine,
+			"input.http_proxy_port", &entry))
+	{
+		entry.num_value = gconf_client_get_int (gc,
+				"/system/http_proxy/port", NULL);
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
+
+	memset (&entry, 0, sizeof (entry));
+	if (gconf_client_get_bool (gc, "/system/http_proxy/use_authentication", NULL) == FALSE)
+	{
+		if (xine_config_lookup_entry (bvw->priv->xine,
+				"input.http_proxy_user", &entry))
+		{
+			entry.str_value = "";
+			xine_config_update_entry (bvw->priv->xine, &entry);
+		}
+
+		memset (&entry, 0, sizeof (entry));
+		if (xine_config_lookup_entry (bvw->priv->xine,
+				"input.http_proxy_password", &entry))
+		{
+			entry.str_value = "";
+			xine_config_update_entry (bvw->priv->xine, &entry);
+		}
+	} else {
+		if (xine_config_lookup_entry (bvw->priv->xine,
+				"input.http_proxy_user", &entry))
+		{
+			entry.str_value = gconf_client_get_string (gc,
+					"/system/http_proxy/authentication_user",
+					NULL);
+			xine_config_update_entry (bvw->priv->xine, &entry);
+		}
+
+		memset (&entry, 0, sizeof (entry));
+		if (xine_config_lookup_entry (bvw->priv->xine,
+				"input.http_proxy_password", &entry))
+		{
+			entry.str_value = gconf_client_get_string (gc,
+					"/system/http_proxy/authentication_password",
+					NULL);
+			xine_config_update_entry (bvw->priv->xine, &entry);
+		}
+	}
 }
 
 static void
@@ -913,8 +952,6 @@ bacon_video_widget_realize (GtkWidget *widget)
 			"configure-event",
 			G_CALLBACK (configure_cb), bvw);
 
-	bvw->priv->scr = scrsaver_new (GDK_DISPLAY ());
-
 	/* Now onto the video out driver */
 	bvw->priv->display = XOpenDisplay (gdk_display_get_name
 			(gdk_display_get_default ()));
@@ -938,11 +975,6 @@ bacon_video_widget_realize (GtkWidget *widget)
 	xine_event_create_listener_thread (bvw->priv->ev_queue,
 			xine_event, (void *) bvw);
 
-	/* Setup the default screen stuff */
-	update_fullscreen_size (bvw);
-	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
-			"size-changed", G_CALLBACK (size_changed_cb), bvw);
-
 	return;
 }
 
@@ -994,6 +1026,66 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 	return (queue_length > 0);
 }
 
+static void
+xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
+{
+	char *message;
+
+	message = NULL;
+
+	if (!(data->explanation))
+	{
+		D("xine_event_message: UI message without an explanation\n"
+				"type: %d", msg->type);
+		return;
+	}
+
+	switch(data->type)
+	{
+	case XINE_MSG_NO_ERROR:
+		break;
+	case XINE_MSG_GENERAL_WARNING:
+		break;
+	case XINE_MSG_UNKNOWN_HOST:
+		message = g_strdup_printf (_("The server you are trying to connect to (%s) is not known."), (char *) data + data->parameters);
+		break;
+	case XINE_MSG_UNKNOWN_DEVICE:
+		message = g_strdup_printf (_("The device name you specified (%s) seems to be invalid."), (char *) data + data->parameters);
+		break;
+	case XINE_MSG_NETWORK_UNREACHABLE:
+		message = g_strdup_printf (_("The server you are trying to connect to (%s) is unreachable."), (char *) data + data->parameters);
+		break;
+	case XINE_MSG_CONNECTION_REFUSED:
+		message = g_strdup_printf (_("The connection to this server was refused."));
+		break;
+	case XINE_MSG_FILE_NOT_FOUND:
+		message = g_strdup_printf (_("The specified movie '%s' could not be found."), (char *) data + data->parameters);
+		break;
+	case XINE_MSG_READ_ERROR:
+		message = g_strdup_printf (_("The movie '%s' could not be read."), (char *) data + data->parameters);
+		break;
+	case XINE_MSG_LIBRARY_LOAD_ERROR:
+		message = g_strdup_printf (_("A problem occur while loading a library or a decoder (%s)."), (char *) data + data->parameters);
+		break;
+	case XINE_MSG_ENCRYPTED_SOURCE:
+		message = g_strdup_printf (_("The source seems encrypted, and can't be read. Are you trying to play an encrypted DVD without libdvdcss?"));
+		break;
+	}
+
+	if (message == NULL)
+	{
+		D("xine_event_message: unhandled error\n"
+				"type: %d", msg->type);
+		return;
+	}
+
+	g_signal_emit (G_OBJECT (bvw),
+			bvw_table_signals[ERROR], 0,
+			message);
+
+	g_free (message);
+}
+
 static void
 xine_event (void *user_data, const xine_event_t *event)
 {
@@ -1037,9 +1129,10 @@ xine_event (void *user_data, const xine_event_t *event)
 		break;
 	case XINE_EVENT_MRL_REFERENCE:
 		ref = event->data;
-
-		//FIXME
-		g_message ("ref mrl detected: %s", ref->mrl);
+		D("ref mrl detected: %s", ref->mrl);
+		break;
+	case XINE_EVENT_UI_MESSAGE:
+		xine_event_message (bvw, (xine_ui_message_data_t *)event->data);
 		break;
 	}
 }
@@ -1068,6 +1161,9 @@ xine_error (BaconVideoWidget *bvw, GError **error)
 		g_set_error (error, 0, 0, _("This location is not "
 					"a valid one"));
 		break;
+	case XINE_ERROR_INPUT_FAILED:
+		g_set_error (error, 0, 0, _("This movie could not be opened"));
+		break;
 	default:
 		g_set_error (error, 0, 0, _("Generic Error"));
 		break;
@@ -1086,9 +1182,6 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 
 	bvw = BACON_VIDEO_WIDGET (widget);
 
-	/* Kill the Screesaver */
-	scrsaver_free (bvw->priv->scr);
-
 	/* Kill the TV out */
 	xine_tvmode_exit (bvw->priv->xine);
 
@@ -1128,10 +1221,6 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	xine_exit (bvw->priv->xine);
 	bvw->priv->xine = NULL;
 
-	/* Finally, kill the left-over windows */
-	if (bvw->priv->fullscreen_window != NULL)
-		gdk_window_destroy (bvw->priv->fullscreen_window);
-
 	/* This destroys widget->window and unsets the realized flag */
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		(*GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
@@ -1244,6 +1333,28 @@ bacon_video_widget_key_press (GtkWidget *widget, GdkEventKey *event)
 	return FALSE;
 }
 
+
+static void
+bacon_video_widget_size_request (GtkWidget *widget, GtkRequisition *requisition)
+{
+	BaconVideoWidget *bvw;
+	GtkRequisition child_requisition;
+	
+	g_return_if_fail(widget != NULL);
+	g_return_if_fail(BACON_IS_VIDEO_WIDGET(widget));
+
+	bvw = BACON_VIDEO_WIDGET (widget);
+	
+	if ((bvw->priv->init_width == 0) && (bvw->priv->init_height == 0)) {
+		requisition->width = DEFAULT_WIDTH;
+		requisition->height = DEFAULT_HEIGHT;
+	} else {
+		/* Requesting first allocation as a minimum */
+		requisition->width = bvw->priv->init_width;
+		requisition->height = bvw->priv->init_height;
+	}
+}
+
 static void
 bacon_video_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -1257,6 +1368,12 @@ bacon_video_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	widget->allocation = *allocation;
 	bvw->priv->xpos = allocation->x;
 	bvw->priv->ypos = allocation->y;
+	
+	if ( (bvw->priv->init_width == 0) && (bvw->priv->init_height == 0) ) {
+		/* First allocation, saving values */
+		bvw->priv->init_width = allocation->width;
+		bvw->priv->init_height = allocation->height;
+	}
 
 	if (GTK_WIDGET_REALIZED (widget))
 	{
@@ -1517,10 +1634,6 @@ bacon_video_widget_set_property (GObject *object, guint property_id,
 		bacon_video_widget_set_logo_mode (bvw,
 				g_value_get_boolean (value));
 		break;
-	case PROP_FULLSCREEN:
-		bacon_video_widget_set_fullscreen (bvw,
-				g_value_get_boolean (value));
-		break;
 	case PROP_SPEED:
 		bacon_video_widget_set_speed (bvw, g_value_get_int (value));
 		break;
@@ -1557,10 +1670,6 @@ bacon_video_widget_get_property (GObject *object, guint property_id,
 		g_value_set_boolean (value,
 				bacon_video_widget_get_logo_mode (bvw));
 		break;
-	case PROP_FULLSCREEN:
-		g_value_set_boolean (value,
-				bacon_video_widget_is_fullscreen (bvw));
-		break;
 	case PROP_SPEED:
 		g_value_set_int (value, bacon_video_widget_get_speed (bvw));
 		break;
@@ -1674,93 +1783,6 @@ bacon_video_widget_get_position (BaconVideoWidget *bvw)
 	return pos_stream;
 }
 
-void
-bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
-{
-	g_return_if_fail (bvw != NULL);
-	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-	g_return_if_fail (bvw->priv->xine != NULL);
-
-	if (bvw->priv->pml == FALSE)
-		bvw->priv->pml = TRUE;
-	else
-		return;
-
-	if (fullscreen == bvw->priv->fullscreen_mode)
-		return;
-
-	bvw->priv->fullscreen_mode = fullscreen;
-
-	if (fullscreen)
-	{
-		GdkWindow *parent;
-		GdkWindowAttr attr;
-
-		parent = gdk_window_get_toplevel (GTK_WIDGET(bvw)->window);
-
-		update_fullscreen_size (bvw);
-
-		attr.x = bvw->priv->fullscreen_rect.x;
-		attr.y = bvw->priv->fullscreen_rect.y;
-		attr.width = bvw->priv->fullscreen_rect.width;
-		attr.height = bvw->priv->fullscreen_rect.height;
-		attr.window_type = GDK_WINDOW_TOPLEVEL;
-		attr.wclass = GDK_INPUT_OUTPUT;
-		attr.override_redirect = FALSE;
-		attr.event_mask = gtk_widget_get_events (GTK_WIDGET (bvw))
-			| GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK
-			| GDK_BUTTON_PRESS_MASK | GDK_KEY_PRESS_MASK;
-		bvw->priv->fullscreen_window = gdk_window_new
-			(NULL, &attr, GDK_WA_X | GDK_WA_Y | GDK_WA_NOREDIR);
-		gdk_window_show (bvw->priv->fullscreen_window);
-		gdk_window_fullscreen (bvw->priv->fullscreen_window);
-		/* Flush, so that the window is really shown */
-		gdk_flush ();
-
-		gdk_window_set_user_data (bvw->priv->fullscreen_window, bvw);
-
-		xine_gui_send_vo_data (bvw->priv->stream,
-			 XINE_GUI_SEND_DRAWABLE_CHANGED,
-			 (void*) GDK_WINDOW_XID (bvw->priv->fullscreen_window));
-
-		/* switch off mouse cursor */
-		bacon_video_widget_set_show_cursor (bvw, FALSE);
-
-		scrsaver_disable (bvw->priv->scr);
-
-		/* Set the icon and the name */
-		if (bvw->priv->icon != NULL)
-		{
-			GList *list = NULL;
-
-			list = g_list_append (list, bvw->priv->icon);
-			gdk_window_set_icon_list (bvw->priv->fullscreen_window,
-					list);
-			g_list_free (list);
-		}
-		gdk_window_set_title (bvw->priv->fullscreen_window,
-				DEFAULT_TITLE);
-	} else {
-		gdk_window_set_user_data (GTK_WIDGET(bvw)->window, bvw);
-
-		xine_gui_send_vo_data (bvw->priv->stream,
-			 XINE_GUI_SEND_DRAWABLE_CHANGED,
-			 (void *) GDK_WINDOW_XID (bvw->priv->video_window));
-
-		/* Hide the window */
-		gdk_window_destroy (bvw->priv->fullscreen_window);
-		bvw->priv->fullscreen_window = NULL;
-
-		scrsaver_enable (bvw->priv->scr);
-
-		gdk_window_focus (gdk_window_get_toplevel
-				(gtk_widget_get_parent_window
-				 (GTK_WIDGET (bvw))), GDK_CURRENT_TIME);
-	}
-
-	bvw->priv->pml = FALSE;
-}
-
 gboolean
 bacon_video_widget_is_fullscreen (BaconVideoWidget *bvw)
 {
@@ -1828,6 +1850,11 @@ bacon_video_widget_get_volume (BaconVideoWidget *bvw)
 	return bvw->priv->volume;
 }
 
+void
+bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
+{
+}
+
 void
 bacon_video_widget_set_show_cursor (BaconVideoWidget *bvw, gboolean show_cursor)
 {
@@ -1835,15 +1862,11 @@ bacon_video_widget_set_show_cursor (BaconVideoWidget *bvw, gboolean show_cursor)
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
-	if (GDK_IS_WINDOW (bvw->priv->fullscreen_window) == FALSE)
-		return;
-
 	if (show_cursor == FALSE)
 	{
-		eel_gdk_window_set_invisible_cursor
-			(bvw->priv->fullscreen_window);
+		eel_gdk_window_set_invisible_cursor (bvw->priv->video_window);
 	} else {
-		gdk_window_set_cursor (bvw->priv->fullscreen_window, NULL);
+		gdk_window_set_cursor (bvw->priv->video_window, NULL);
 	}
 
 	bvw->priv->cursor_shown = show_cursor;
@@ -2142,6 +2165,16 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget *bvw,
 	}
 }
 
+gboolean
+bacon_video_widget_get_auto_resize (BaconVideoWidget *bvw)
+{
+	g_return_if_fail (bvw != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+	g_return_if_fail (bvw->priv->xine != NULL);
+
+	return bvw->priv->auto_resize;
+}
+
 void
 bacon_video_widget_set_auto_resize (BaconVideoWidget *bvw, gboolean auto_resize)
 {
@@ -2284,6 +2317,7 @@ bacon_video_widget_toggle_aspect_ratio (BaconVideoWidget *bvw)
 static gboolean
 bacon_video_widget_ratio_fits_screen (BaconVideoWidget *bvw, gfloat ratio)
 {
+	GdkRectangle fullscreen_rect;
 	int new_w, new_h;
 
 	g_return_val_if_fail (bvw != NULL, FALSE);
@@ -2293,10 +2327,14 @@ bacon_video_widget_ratio_fits_screen (BaconVideoWidget *bvw, gfloat ratio)
 	new_w = bvw->priv->video_width * ratio;
 	new_h = bvw->priv->video_height * ratio;
 
-	update_fullscreen_size (bvw);
+	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
+			gdk_screen_get_monitor_at_window
+			(gdk_screen_get_default (),
+			 bvw->priv->video_window),
+			&fullscreen_rect);
 
-	if (new_w > (bvw->priv->fullscreen_rect.width - 128) ||
-			new_h > (bvw->priv->fullscreen_rect.height - 128))
+	if (new_w > (fullscreen_rect.width - 128) ||
+			new_h > (fullscreen_rect.height - 128))
 	{
 		return FALSE;
 	}
@@ -2307,7 +2345,7 @@ bacon_video_widget_ratio_fits_screen (BaconVideoWidget *bvw, gfloat ratio)
 void
 bacon_video_widget_set_scale_ratio (BaconVideoWidget *bvw, gfloat ratio)
 {
-	GtkWidget *toplevel;
+	GtkWidget *toplevel, *widget;
 	int new_w, new_h, win_w, win_h;
 
 	g_return_if_fail (bvw != NULL);
@@ -2340,18 +2378,30 @@ bacon_video_widget_set_scale_ratio (BaconVideoWidget *bvw, gfloat ratio)
 			return;
 	}
 
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (bvw));
+	widget = GTK_WIDGET (bvw);
+
+	toplevel = gtk_widget_get_toplevel (widget);
 
 	/* Get the size of the toplevel window */
-	gdk_drawable_get_size (GDK_DRAWABLE (GTK_WIDGET (toplevel)->window),
+	gdk_drawable_get_size (GDK_DRAWABLE (toplevel->window),
 			&win_w, &win_h);
 
 	/* Calculate the new size of the window, depending on the size of the
 	 * video widget, and the new size of the video */
-	new_w = win_w - GTK_WIDGET(bvw)->allocation.width +
-		(bvw->priv->video_width * ratio);
-	new_h = win_h - GTK_WIDGET(bvw)->allocation.height +
-		(bvw->priv->video_height * ratio);
+	new_w = win_w - widget->allocation.width +
+		bvw->priv->video_width * ratio;
+	new_h = win_h - widget->allocation.height +
+		bvw->priv->video_height * ratio;
+
+	/* Change the minimum size of the widget
+	 * but only if we're getting a smaller window */
+	if (new_w < widget->allocation.width
+			|| new_h < widget->allocation.height)
+	{
+		gtk_widget_set_size_request (widget,
+				bvw->priv->video_width * ratio,
+				bvw->priv->video_height * ratio);
+	}
 
 	gtk_window_resize (GTK_WINDOW (toplevel), new_w, new_h);
 }
