@@ -101,7 +101,8 @@ on_checkbutton1_toggled (GtkToggleButton *togglebutton, Totem *totem)
 
 static void              
 on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
-{                               
+{
+	GtkWidget *item;
 	gboolean value;
 
 	value = gtk_toggle_button_get_active (togglebutton);
@@ -118,6 +119,16 @@ on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
 	}
 
 	gconf_client_set_bool (totem->gc, GCONF_PREFIX"/show_vfx", value, NULL);
+
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_label");
+	gtk_widget_set_sensitive (item, value);
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_optionmenu");
+	gtk_widget_set_sensitive (item, value);
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_label");
+	gtk_widget_set_sensitive (item, value);
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_optionmenu");
+	gtk_widget_set_sensitive (item, value);
+
 	if (bacon_video_widget_set_show_visuals
 		(BACON_VIDEO_WIDGET (totem->bvw), value) == FALSE)
 	{
@@ -164,7 +175,7 @@ deinterlace_changed_cb (GConfClient *client, guint cnxn_id,
 {
 	GtkWidget *item;
 
-	item = glade_xml_get_widget (totem->xml, "deinterlace1");
+	item = glade_xml_get_widget (totem->xml, "tmw_deinterlace_menu_item");
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
 			on_deinterlace1_activate, totem);
 
@@ -193,7 +204,7 @@ auto_resize_changed_cb (GConfClient *client, guint cnxn_id,
 {
 	GtkWidget *item;
 
-	item = glade_xml_get_widget (totem->xml, "checkbutton1");
+	item = glade_xml_get_widget (totem->xml, "tpw_display_checkbutton");
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
 			on_checkbutton1_toggled, totem);
 
@@ -211,7 +222,7 @@ show_vfx_changed_cb (GConfClient *client, guint cnxn_id,
 {
 	GtkWidget *item;
 
-	item = glade_xml_get_widget (totem->xml, "checkbutton2");
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_checkbutton");
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
 			on_checkbutton2_toggled, totem);
 
@@ -230,7 +241,7 @@ mediadev_changed_cb (GConfClient *client, guint cnxn_id,
 	GtkWidget *item;
 	char *mediadev;
 
-	item = glade_xml_get_widget (totem->xml, "custom3");
+	item = glade_xml_get_widget (totem->xml, "tpw_device_combo");
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
 			on_combo_entry1_changed, totem);
 
@@ -266,8 +277,6 @@ on_button1_clicked (GtkButton *button, Totem *totem)
 
 	path = path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
-	if (g_file_test (path, G_FILE_TEST_IS_DIR) == FALSE)
-		mkdir (path, 0775);
 
 	cmd = g_strdup_printf ("nautilus --no-default-window %s", path);
 	g_free (path);
@@ -285,6 +294,42 @@ on_button1_clicked (GtkButton *button, Totem *totem)
 	g_free (cmd);
 }
 
+static void
+visual_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
+{
+	GList *list;
+	const char *old_name;
+	char *name;
+	int i;
+
+	i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+	list = bacon_video_widget_get_visuals_list (totem->bvw);
+	name = g_list_nth_data (list, i);
+
+	old_name = gconf_client_get_string (totem->gc,
+			GCONF_PREFIX"/visual", NULL);
+
+	if (old_name == NULL || strcmp (old_name, name) != 0)
+	{
+		gconf_client_set_string (totem->gc, GCONF_PREFIX"/visual",
+				name, NULL);
+
+		if (bacon_video_widget_set_visuals (totem->bvw, name) == TRUE)
+			totem_action_error (_("Changing the visuals effect type will require a restart to take effect."), totem);
+	}
+}
+
+static void
+visual_quality_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
+{
+	int i;
+
+	i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+	gconf_client_set_int (totem->gc,
+			GCONF_PREFIX"/visual_quality", i, NULL);
+	bacon_video_widget_set_visuals_quality (totem->bvw, i);
+}
+
 GtkWidget *
 bacon_cd_selection_create (void)
 {
@@ -299,11 +344,12 @@ bacon_cd_selection_create (void)
 void
 totem_setup_preferences (Totem *totem)
 {
-	GtkWidget *item;
+	GtkWidget *item, *menu;
 	const char *mediadev;
 	gboolean show_visuals, auto_resize, is_local, deinterlace;
-	int connection_speed;
-	char *path;
+	int connection_speed, i, found;
+	char *path, *visual;
+	GList *list, *l;
 
 	g_return_if_fail (totem->gc != NULL);
 
@@ -321,36 +367,25 @@ totem_setup_preferences (Totem *totem)
 			(GConfClientNotifyFunc) mediadev_changed_cb,
 			totem, NULL, NULL);
 
-	totem->prefs = glade_xml_get_widget (totem->xml, "dialog1");
+	totem->prefs = glade_xml_get_widget (totem->xml, "totem_preferences_window");
 
 	g_signal_connect (G_OBJECT (totem->prefs),
 			"response", G_CALLBACK (hide_prefs), (gpointer) totem);
 	g_signal_connect (G_OBJECT (totem->prefs), "delete-event",
 			G_CALLBACK (hide_prefs), (gpointer) totem);
 
+	/* Auto-resize */
 	auto_resize = gconf_client_get_bool (totem->gc,
 			GCONF_PREFIX"/auto_resize", NULL);
-	item = glade_xml_get_widget (totem->xml, "checkbutton1");
+	item = glade_xml_get_widget (totem->xml, "tpw_display_checkbutton");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item), auto_resize);
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_checkbutton1_toggled), totem);
 	bacon_video_widget_set_auto_resize
 		(BACON_VIDEO_WIDGET (totem->bvw), auto_resize);
 
-	item = glade_xml_get_widget (totem->xml, "checkbutton2");
-	show_visuals = gconf_client_get_bool (totem->gc,
-			GCONF_PREFIX"/show_vfx", NULL);
-	if (is_local == FALSE && show_visuals == TRUE)
-		show_visuals = ask_show_visuals (totem);
-
-	gtk_toggle_button_set_active
-		(GTK_TOGGLE_BUTTON (item), show_visuals);
-	bacon_video_widget_set_show_visuals
-		(BACON_VIDEO_WIDGET (totem->bvw), show_visuals);
-	g_signal_connect (G_OBJECT (item), "toggled",
-			G_CALLBACK (on_checkbutton2_toggled), totem);
-
-	item = glade_xml_get_widget (totem->xml, "custom3");
+	/* Media device */
+	item = glade_xml_get_widget (totem->xml, "tpw_device_combo");
 	mediadev = gconf_client_get_string
 		(totem->gc, GCONF_PREFIX"/mediadev", NULL);
 	if (mediadev == NULL || (strcmp (mediadev, "") == 0)
@@ -373,25 +408,82 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "device-changed",
 			G_CALLBACK (on_combo_entry1_changed), totem);
 
+	/* Connection Speed */
 	connection_speed = bacon_video_widget_get_connection_speed (totem->bvw);
-	item = glade_xml_get_widget (totem->xml, "optionmenu1");
+	item = glade_xml_get_widget (totem->xml, "tpw_speed_optionmenu");
 	gtk_option_menu_set_history (GTK_OPTION_MENU (item),
 			connection_speed);
 	g_signal_connect (item, "changed",
-			(GCallback)option_menu_connection_changed, totem);
+			G_CALLBACK (option_menu_connection_changed), totem);
 
-	item = glade_xml_get_widget (totem->xml, "button1");
+	/* Proprietary plugins */
+	item = glade_xml_get_widget (totem->xml, "tpw_plugins_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
 			G_CALLBACK (on_button1_clicked), totem);
 	path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
-	bacon_video_widget_set_proprietary_plugins_path
-		(totem->bvw, path);
+	if (g_file_test (path, G_FILE_TEST_IS_DIR) == FALSE)
+		mkdir (path, 0775);
+	bacon_video_widget_set_proprietary_plugins_path (totem->bvw, path);
 	g_free (path);
 
+	/* Enable visuals */
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_checkbutton");
+	show_visuals = gconf_client_get_bool (totem->gc,
+			GCONF_PREFIX"/show_vfx", NULL);
+	if (is_local == FALSE && show_visuals == TRUE)
+		show_visuals = ask_show_visuals (totem);
+
+	gtk_toggle_button_set_active
+		(GTK_TOGGLE_BUTTON (item), show_visuals);
+	bacon_video_widget_set_show_visuals
+		(BACON_VIDEO_WIDGET (totem->bvw), show_visuals);
+	g_signal_connect (G_OBJECT (item), "toggled",
+			G_CALLBACK (on_checkbutton2_toggled), totem);
+
+	/* Visuals list */
+	list = bacon_video_widget_get_visuals_list (totem->bvw);
+	menu = gtk_menu_new ();
+	gtk_widget_show (menu);
+
+	visual = gconf_client_get_string (totem->gc,
+			GCONF_PREFIX"/visual", NULL);
+	if (visual == NULL || strcmp (visual, "") == 0)
+		visual = g_strdup ("goom");
+
+	i = 0;
+	for (l = list; l != NULL; l = l->next)
+	{
+		const char *name = l->data;
+
+		item = gtk_menu_item_new_with_label (name);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
+
+		if (strcmp (name, visual) == 0)
+			found = i;
+
+		i++;
+	}
+
+	/* Visualisation quality */
+	i = gconf_client_get_int (totem->gc,
+			GCONF_PREFIX"/visual_quality", NULL);
+	bacon_video_widget_set_visuals_quality (totem->bvw, i);
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_optionmenu");
+	gtk_option_menu_set_history (GTK_OPTION_MENU (item), i);
+	g_signal_connect (G_OBJECT (item), "changed",
+			G_CALLBACK (visual_quality_menu_changed), totem);
+
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_optionmenu");
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (item), menu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (item), found);
+	g_signal_connect (G_OBJECT (item), "changed",
+			G_CALLBACK (visual_menu_changed), totem);
+
 	/* This one is for the deinterlacing menu, not really our dialog
 	 * but we do it anyway */
-	item = glade_xml_get_widget (totem->xml, "deinterlace1");
+	item = glade_xml_get_widget (totem->xml, "tmw_deinterlace_menu_item");
 	deinterlace = gconf_client_get_bool (totem->gc,
 			GCONF_PREFIX"/deinterlace", NULL);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
@@ -415,13 +507,13 @@ totem_preferences_tvout_setup (Totem *totem)
 	switch (type)
 	{
 	case TV_OUT_NONE:
-		name = "notvout";
+		name = "tpw_notvout_radio_button";
 		break;
 	case TV_OUT_TVMODE:
-		name = "tvoutmode";
+		name = "tpw_tvoutmode_radio_button";
 		break;
 	case TV_OUT_DXR3:
-		name = "dxr3tvout";
+		name = "tpw_dxr3tvout_radio_button";
 		break;
 	default:
 		g_assert_not_reached ();
@@ -430,20 +522,32 @@ totem_preferences_tvout_setup (Totem *totem)
 	item = glade_xml_get_widget (totem->xml, name);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item), TRUE);
 
-	item = glade_xml_get_widget (totem->xml, "notvout");
+	item = glade_xml_get_widget (totem->xml, "tpw_notvout_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_NONE));
-	item = glade_xml_get_widget (totem->xml, "tvoutmode");
+	item = glade_xml_get_widget (totem->xml, "tpw_tvoutmode_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_TVMODE));
-	item = glade_xml_get_widget (totem->xml, "dxr3tvout");
+	item = glade_xml_get_widget (totem->xml, "tpw_dxr3tvout_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_DXR3));
 }
 
+void
+totem_preferences_visuals_setup (Totem *totem)
+{
+	char *visual;
+
+	visual = gconf_client_get_string (totem->gc,
+			GCONF_PREFIX"/visual", NULL);
+	if (visual == NULL || strcmp (visual, "") == 0)
+		visual = g_strdup ("goom");
+
+	bacon_video_widget_set_visuals (totem->bvw, visual);
+}
